#
# Simple SER config script for my testdomain biloxi.com (F. Edler private)
# functions successfully tested:
# - xlog
# - nat_flag for registrar database (dafault=4)
# - fix_nated_contact
# - force_rport
# - t_on_reply and onrepy_route
# -
# -
# -
#
# ----------- global configuration parameters ------------------------

debug=3         	# debug level (0..9), default = 0, (cmd line: -dddddddddd)
fork=no 		# enable server to spawn children, no helps at debugging, default = yes
log_stderror=yes	# yes causes debugging info at standard output, on syslog otherwise, default = no, (cmd line: -E)

listen=sip.biloxi.com	# IP address(es) or hostnames where SER will listen for messages
#port=5060		# IP port where SER will listen for messages, default=5060
alias=biloxi.com	# additional IP address(es) or hostnames to satisfy the condition "uri==myself"


check_via=no		# enable Via host checking when forwarding replies, default=no (cmd. line: -v)
dns=no           	# enable DNS check for adding "received" field in via, default=no (cmd. line: -r)
rev_dns=no      	# enable reverse DNS check for adding "received" field in via, default=no(cmd. line: -R)
#sip_warning=no		# enables warning header in replies, good for debugging, default=yes
#server_signature=no	# enables "server"-header for local generated messages, good for debugging, default=yes
#reply_to_via=yes	# send replies to address advertised in Via or to IP-address of request, default=no

#children=2		# determine number of processes (in case of fork) in parallel for each transport protocol
fifo="/tmp/ser_fifo"	# special file (pipe) to enable communication with serweb and serctl
fifo_mode=0666		# enables access to fifo via different user (serweb on apache)

# ------------------ module loading ----------------------------------

loadmodule "/usr/local/lib/ser/modules/mysql.so"	# for SQL database
loadmodule "/usr/local/lib/ser/modules/sl.so"		# no dependencies on other SER modules
loadmodule "/usr/local/lib/ser/modules/tm.so"		# no dependencies on other SER modules
loadmodule "/usr/local/lib/ser/modules/rr.so"		# no dependencies on other SER modules
loadmodule "/usr/local/lib/ser/modules/maxfwd.so"	# no dependencies on other SER modules
loadmodule "/usr/local/lib/ser/modules/usrloc.so"	# requires optionally a database-module (e.g. mysql.so)
loadmodule "/usr/local/lib/ser/modules/registrar.so"	# requires usrloc.so and sl.so
loadmodule "/usr/local/lib/ser/modules/textops.so"	# no dependencies on other SER modules
# loadmodule "/usr/local/lib/ser/modules/exec.so"	# no dependencies on other SER modules
loadmodule "/usr/local/lib/ser/modules/uri.so"		# requires a database-module (e.g. mysql.so)
# loadmodule "/usr/local/lib/ser/modules/acc.so"	# requires tm.so and a database-module (e.g. mysql.so)
loadmodule "/usr/local/lib/ser/modules/auth.so"		# requires sl.so (for stateless replies)
loadmodule "/usr/local/lib/ser/modules/auth_db.so"	# requires auth.so and a database-module (e.g. mysql.so)
loadmodule "/usr/local/lib/ser/modules/vm.so"		# requires tm.so and a database-module (e.g. mysql.so)
loadmodule "/usr/local/lib/ser/modules/xlog.so"		# no dependencies on other SER modules
loadmodule "/usr/local/lib/ser/modules/nathelper.so"	# no dependencies on other SER modules

# ----------------- setting module-specific parameters ---------------

# -- usrloc params --
modparam("usrloc", "db_mode", 2)			# for persistent storage (write back based on a timer)

# -- auth params --
modparam("auth_db", "calculate_ha1", yes)		# use plaintext password in database
modparam("auth_db", "password_column", "password")	# name of password column

# -- rr params --
modparam("rr", "enable_full_lr", 1)			# add value to ;lr param to make some broken UAs happy

# -- acc params --
# modparam("acc", "log_fmt", "mfs")			# define headfields in log records
# modparam("acc", "log_level", 2)			# log level defines level to issues records to syslog
# modparam("acc", "log_flag", 1)			# request flag to mark a transaction for accounting
# modparam("acc", "log_missed_flag", 3)			# request flag to record missed calls for accounting
# modparam("acc", "db_flag", 2)				# request flag to mark a transaction for accounting (db-specific)
# modparam("acc", "db_missed_flag", 3)			# request flag to record missed calls for accounting (db-spec.)

# -- nathelper params --
# modparam("nathelper", "natping_interval", 60)		# ping intervall in sec.
# modparam("nathelper", "ping_nated_only", 1)		# ping nur an UAs registered as behind NAT

# -------------------------  request routing logic -------------------

# main routing logic
route
{	# initial sanity check -- stop messages with max_forwards==0
	# max_forwards==0, or excessively long requests
	if (!mf_process_maxfwd_header("10"))
	{
		sl_send_reply("483","Too Many Hops");
		break;
	};
	# initial sanity check -- stop excessively long requests
	if (msg:len > max_len)
	{
		sl_send_reply("513", "Message too big");
		break;
	};

	# record-route all messages -- to make sure that
	# subsequent messages will go through our proxy; that's
	# particularly good if upstream and downstream entities
	# use different transport protocol

	#debug info
	xlog("L_INFO","[%rm] from [%fu] to [%tu] r_uri [%ru]\n");
	xlog("L_INFO","[%rm] contact [%ct]\n");

	# loose-route processing (grant Route routing if route headers present)
	if (loose_route())
	{
		t_relay();
		break;
	};

	if (nat_uac_test("2"))
	{
		log(1,"src_IP different to Via_IP: force_rport and fix_nated_contact\n");
		force_rport();
		fix_nated_contact();
		append_hf("P-hint: force_rport and fix_nated_contact\r\n");
		# nat_flag for registrar-database: default=4
		setflag(5);
	};

	# check if request-URI is an alias
	lookup ("aliases");

	# handling of messages for domain(s), where SER is responsible
	if (uri==myself)
	{
		# handling of REGISTER requests 
		if (method=="REGISTER")
		{
			# credentials required
			if (!www_authorize("biloxi.com", "subscriber"))
			{
				www_challenge("biloxi.com", "0");
				break;
			};
			# check match of username in To-header against username in URI
			if (!check_to())
			{
				sl_send_reply("403", "Forbidden - credentials unacceptable");
				break;
			};
			if isflagset(5)
			{
				# mark UA behind NAT in database (location database flag=1)
				log(1,"Registration behind NAT\n");
				setflag(4);
			}
			else
			{
				log(1,"Registration not behind NAT\n");
			};
			# now update location database
			save("location");
			break;
		};

		# interim patch to make snom happy:
		# stop PUBLISH (used for number guessing) and SUBSCRIBE requests from "snom" 
		if (((method=="SUBSCRIBE")||(method=="PUBLISH")) && (search("product=snom200")))
		{
			sl_send_reply("501", "Not Inplemented");
			break;
		};

		# lookup in user location database
		# is uri in database
		if (!does_uri_exist())
		{
			sl_send_reply("404", "Sorry, user unknown");
			break;
		} else
		{
			# is user actually registered
			if (!lookup("location"))
			{
				sl_send_reply("480", "Sorry, temporarily unavailable");
				break;
			} else	# o.k. forward request (=inbound proxy)
			{
				route(1);
			}
		};
	} else
	{
		# request is for foreign domain (SER not responsible) or UA direct (no lookup)
		route(1);
	};
}

route[1]
{
	# Is one side of session (incoming Request or UAS) behind NAT ?
	if (isflagset(5) || isflagset(4))
	{
		#debug info
		if isflagset(5)
		{
			log(1,"incoming request from behind NAT\n");
		}
		else
		{
			log(1,"UAS is behind NAT\n");
		};
		# enable reply-processing for nated sessions
		t_on_reply("1");
		if (method==INVITE)
		{
			# record route necessary to enable proxy to modify requests and responses
			record_route();
			# force session via RTP-proxy
			log(1,"INVITE requires RTP-Proxy\n");
			force_rtp_proxy();
			append_hf("P-hint: session forced via RTP-Proxy\r\n");
			# mark INVITE transaction for reply-processing (force_rtp_proxy)
			setflag(6);
		};
		# go on and forward request
		if (!t_relay())
		{
			sl_reply_error();
			break;
		};
	};
}

onreply_route[1]
{
	xlog("L_INFO","Reply: status [%rs] reason [%rr]\n");
	# transaction to UAS behind NAT?
	if setflag(4)
	{
		log(1,"UAS behind NAT: fix Contact header in Reply\n");
		fix_nated_contact();
		append_hf("P-hint: Contact header fixed in Reply\r\n");
		# INVITE transaction?
		if isflagset(6)
		{
			if (status=~"183" || status=~"200")
			{
			log(1,"positive INVITE Reply: force RTP-Proxy\n");
			force_rtp_proxy();
			append_hf("P-hint: Reply forced via RTP-Proxy\r\n");
			}
		};
	};
}



